library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity Nexys_comm is
    port (
        mclk : in std_logic ;
        pdb : inout std_logic_vector (7 downto 0) ;
        astb : inout std_logic ;
        dstb : inout std_logic ;
        pwr : inout std_logic ;
        pwait : inout std_logic;
        btn : in std_logic_vector (3 downto 0) ;

        sw : in std_logic_vector (7 downto 0) ;

        ld : out std_logic_vector (7 downto 0) ;

        ssg : out std_logic_vector (7 downto 0) ;

        an : out std_logic_vector (3 downto 0)

) ;
end Nexys_comm ;

architecture synthesis of Nexys_comm is

    -- submodules declarations
    component commUSB
        port (
            mclk     : in std_logic;
            pdb      : inout std_logic_vector(7 downto 0);
            astb     : inout std_logic;
            dstb     : inout std_logic;
            pwr      : inout std_logic;
            pwait    : inout std_logic;
            pc2board : out std_logic_vector(127 downto 0);
            board2pc : in std_logic_vector(127 downto 0)
        ) ;
    end component ;
	component Nexys
		port (
			mclk : in std_logic ;
			btn : in std_logic_vector (3 downto 0) ;
			sw : in std_logic_vector (7 downto 0) ;
			ld : out std_logic_vector (7 downto 0) ;
			ssg : out std_logic_vector (7 downto 0) ;
			an : out std_logic_vector (3 downto 0) ;
			txda : out std_logic ;
			pdb : in std_logic_vector (7 downto 0) ;
			astb : in std_logic ;
			dstb : in std_logic ;
			pwr : in std_logic ;
			pwait : out std_logic
		) ;
	end component ;

    -- internal signals declarations
    signal pc2board : std_logic_vector (127 downto 0) ;
    signal board2pc : std_logic_vector (127 downto 0) ;
    signal txda : std_logic ;
    signal pdb : std_logic_vector (7 downto 0) ;
    signal astb : std_logic ;
    signal dstb : std_logic ;
    signal pwr : std_logic ;
    signal pwait : std_logic ;

begin

    -- combinatorial statements
    board2pc(0) <= txda ;
    pdb(7 downto 0) <= pc2board(7 downto 0) ;
    astb <= pc2board(8) ;
    dstb <= pc2board(9) ;
    pwr <= pc2board(10) ;
    board2pc(1) <= pwait ;
	 
	 

    -- component instanciation
    commUSB_0 : commUSB port map (mclk, pdb(7 downto 0), astb, dstb, pwr, pwait, pc2board(127 downto 0), board2pc(127 downto 0)) ;
    Nexys_0 : Nexys port map (mclk, btn(3 downto 0), sw(7 downto 0), ld(7 downto 0), ssg(7 downto 0), an(3 downto 0), txda, pdb(7 downto 0), astb, dstb, pwr, pwait) ;


end synthesis;
