library IEEE;
library UNISIM;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity micromachine is
	port (
		rst : in std_logic ;
		clk : in std_logic ;
		areg : in std_logic_vector (4 downto 0) ;
		breg : in std_logic_vector (4 downto 0) ;
		dreg : in std_logic_vector (4 downto 0) ;
		cmd_ual : in std_logic_vector (5 downto 0) ;
		oe_num : in std_logic_vector (1 downto 0) ;
		write : in std_logic ;
		monitor : in std_logic_vector (31 downto 0) ;
		switch : in std_logic_vector (7 downto 0) ;
		swclk : in std_logic ;
		ssgclk : in std_logic ;
		pwmclk : in std_logic ;
		dbus : inout std_logic_vector (31 downto 0) ;
		n : out std_logic ;
		z : out std_logic ;
		v : out std_logic ;
		c : out std_logic ;
		ir : out std_logic_vector (31 downto 0) ;
		break : out std_logic ;
		leds : out std_logic_vector (7 downto 0) ;
		ssegs : out std_logic_vector (7 downto 0) ;
		anodes : out std_logic_vector (3 downto 0) ;
		pwm_out : out std_logic
	);
end micromachine;

architecture synthesis of micromachine is

	-- submodules declarations
	component decoder2to4
		port (
			e : in std_logic_vector (1 downto 0) ;
			s : out std_logic_vector (3 downto 0)
		) ;
	end component ;
	component ual
		port (
			a : in std_logic_vector (31 downto 0) ;
			b : in std_logic_vector (31 downto 0) ;
			cmd : in std_logic_vector (5 downto 0) ;
			s : inout std_logic_vector (31 downto 0) ;
			setn : out std_logic ;
			setz : out std_logic ;
			setvc : out std_logic ;
			n : out std_logic ;
			z : out std_logic ;
			v : out std_logic ;
			c : out std_logic
		) ;
	end component ;
	component registres
		port (
			rst : in std_logic ;
			clk : in std_logic ;
			areg : in std_logic_vector (4 downto 0) ;
			breg : in std_logic_vector (4 downto 0) ;
			dreg : in std_logic_vector (4 downto 0) ;
			dbus : in std_logic_vector (31 downto 0) ;
			abus : inout std_logic_vector (31 downto 0) ;
			bbus : inout std_logic_vector (31 downto 0) ;
			ir : out std_logic_vector (31 downto 0) ;
			break : out std_logic
		) ;
	end component ;
	component decoder4to16
		port (
			e : in std_logic_vector (3 downto 0) ;
			s : out std_logic_vector (15 downto 0)
		) ;
	end component ;
	component rams_asyn_read512x32
		port (
			clk  : in std_logic;
			we  : in std_logic;
			addr  : in std_logic_vector(8 downto 0);
			di  : in std_logic_vector(31 downto 0);
			do  : out std_logic_vector(31 downto 0)
		) ;
	end component ;
	component reg8
		port (
			rst : in std_logic ;
			clk : in std_logic ;
			cs : in std_logic ;
			d : in std_logic_vector (7 downto 0) ;
			q : out std_logic_vector (7 downto 0)
		) ;
	end component ;
	component segs7
		port (
			rst : in std_logic ;
			clk : in std_logic ;
			ssgclk : in std_logic ;
			en : in std_logic ;
			selec : in std_logic ;
			data : in std_logic_vector (15 downto 0) ;
			anodes : out std_logic_vector (3 downto 0) ;
			ssegs : out std_logic_vector (7 downto 0)
		) ;
	end component ;
	component pwm_module
		port (
			rst : in std_logic ;
			clk : in std_logic ;
			pwmclk : in std_logic ;
			en : in std_logic ;
			selec : in std_logic ;
			data : in std_logic_vector (15 downto 0) ;
			pwm_out : out std_logic
		) ;
	end component ;

	-- buffer signals declarations
	signal ir_int : std_logic_vector (31 downto 0) ;

	-- internal signals declarations
	signal oe : std_logic_vector (3 downto 0) ;
	signal abus : std_logic_vector (31 downto 0) ;
	signal bbus : std_logic_vector (31 downto 0) ;
	signal dbus_ual : std_logic_vector (31 downto 0) ;
	signal setn : std_logic ;
	signal setz : std_logic ;
	signal setvc : std_logic ;
	signal dn : std_logic ;
	signal dz : std_logic ;
	signal dv : std_logic ;
	signal dc : std_logic ;
	signal cs : std_logic_vector (15 downto 0) ;
	signal wrram : std_logic ;
	signal dbus_ram : std_logic_vector (31 downto 0) ;
	signal oe_ram : std_logic ;
	signal swdebounced : std_logic_vector (7 downto 0) ;
	signal oe_sw : std_logic ;
	signal wrssg : std_logic ;
	signal wrld : std_logic ;
	signal wrpwm : std_logic ;

begin

	-- buffer signals assignations
	ir(31 downto 0) <= ir_int(31 downto 0) ;

	-- concurrent statements
	dbus(31 downto 0) <= dbus_ual(31 downto 0) when oe(1) = '1' else "ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ" ;
	wrram <= write and cs(0) ;
	oe_ram <= oe(2) and cs(0) ;
	dbus(31 downto 0) <= dbus_ram(31 downto 0) when oe_ram = '1' else "ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ" ;
	oe_sw <= oe(2) and cs(9) ;
	dbus(31 downto 8) <= "000000000000000000000000" when oe_sw = '1' else "ZZZZZZZZZZZZZZZZZZZZZZZZ" ;
	dbus(7 downto 0) <= swdebounced(7 downto 0) when oe_sw = '1' else "ZZZZZZZZ" ;
	wrssg <= write and cs(10) ;
	wrld <= write and cs(11) ;
	wrpwm <= write and cs(12) ;
	dbus(31 downto 0) <= monitor(31 downto 0) when oe(3) = '1' else "ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ" ;

	-- sequential statements
	process (clk, rst) begin
		if rst = '1' then
			n <= '0' ;
		elsif setn = '1' and clk'event and clk = '1' then
			n <= dn ;
		end if ;
	end process ;
	process (clk, rst) begin
		if rst = '1' then
			z <= '0' ;
		elsif setz = '1' and clk'event and clk = '1' then
			z <= dz ;
		end if ;
	end process ;
	process (clk, rst) begin
		if rst = '1' then
			v <= '0' ;
		elsif setvc = '1' and clk'event and clk = '1' then
			v <= dv ;
		end if ;
	end process ;
	process (clk, rst) begin
		if rst = '1' then
			c <= '0' ;
		elsif setvc = '1' and clk'event and clk = '1' then
			c <= dc ;
		end if ;
	end process ;

	-- components instanciations
	decoder2to4_0 : decoder2to4 port map (oe_num(1 downto 0), oe(3 downto 0)) ;
	ual_1 : ual port map (abus(31 downto 0), bbus(31 downto 0), cmd_ual(5 downto 0), dbus_ual(31 downto 0), setn, setz, setvc, dn, dz, dv, dc) ;
	registres_2 : registres port map (rst, clk, areg(4 downto 0), breg(4 downto 0), dreg(4 downto 0), dbus(31 downto 0), abus(31 downto 0), bbus(31 downto 0), ir_int(31 downto 0), break) ;
	decoder4to16_3 : decoder4to16 port map (abus(31 downto 28), cs(15 downto 0)) ;
	rams_asyn_read512x32_4 : rams_asyn_read512x32 port map (clk, wrram, abus(8 downto 0), dbus(31 downto 0), dbus_ram(31 downto 0)) ;
	reg8_5 : reg8 port map (rst, swclk, '1', switch(7 downto 0), swdebounced(7 downto 0)) ;
	segs7_6 : segs7 port map (rst, clk, ssgclk, wrssg, abus(0), dbus(15 downto 0), anodes(3 downto 0), ssegs(7 downto 0)) ;
	reg8_7 : reg8 port map (rst, clk, wrld, dbus(7 downto 0), leds(7 downto 0)) ;
	pwm_module_8 : pwm_module port map (rst, clk, pwmclk, wrpwm, abus(0), dbus(15 downto 0), pwm_out) ;

end synthesis;
